mkdir k8s
cd k8s

# http://omerio.com/2015/12/18/learn-the-kubernetes-key-concepts-in-10-minutes/
# https://medium.com/google-cloud/kubernetes-nodeport-vs-loadbalancer-vs-ingress-when-should-i-use-what-922f010849e0

# KUBECTL
# https://www.gitbook.com/book/ramitsurana/awesome-kubernetes/details
# https://kubernetes.io/docs/tasks/tools/install-kubectl/
curl -LO https://storage.googleapis.com/kubernetes-release/release/$(curl -s https://storage.googleapis.com/kubernetes-release/release/stable.txt)/bin/linux/amd64/kubectl
chmod +x kubectl

# Paquetes de Ubuntu

# curl -s https://packages.cloud.google.com/apt/doc/apt-key.gpg | apt-key add -
cat <<EOF >/etc/apt/sources.list.d/kubernetes.list
deb http://apt.kubernetes.io/ kubernetes-xenial main
EOF
# apt-get update
# Install docker if you don't have it already.
# apt-get install -y docker-engine
# apt-get install -y kubelet kubeadm kubernetes-cni

# https://github.com/kubernetes/minikube
curl -Lo minikube https://storage.googleapis.com/minikube/releases/latest/minikube-linux-amd64
chmod +x minikube
export PATH=$(pwd):$PATH # alternativa: sudo cp minikube /usr/local/bin/
source <(kubectl completion bash)

# https://www.linux.com/learn/getting-started-kubernetes-easy-minikube
minikube start --memory=4096 --cpus=4
minikube status
kubectl cluster-info
kubectl get nodes
kubectl get pods
kubectl get services
minikube dashboard --url
minikube dashboard


kubectl run hello-minikube --image=gcr.io/google_containers/echoserver:1.4 --port=8080
kubectl expose deployment hello-minikube --type=NodePort
minikube service hello-minikube --url

kubectl delete pod hello-minikube-c8b6b4fdc-j9vh7

kubectl create -f single_container_pod.yaml 
kubectl expose pod web-server --type=NodePort
minikube service web-server
minikube service web-server --url
kubectl delete pod web-server
kubectl delete svc web-server


# Creamos un pod (3 réplicas de nginx)
pod.yaml

apiVersion: apps/v1beta1
kind: Deployment
metadata:
  name: nginx-deployment
spec:
  replicas: 3
  template:
    metadata:
      labels:
        app: nginx
        env: prod
        role: web
    spec:
      containers:
      - name: nginx
        image: nginx:1.7.9
        ports:
        - containerPort: 80
        resources:
          requests:
            cpu: 250m
          limits:
            cpu: 500m

kubectl create -f pod.yaml

kubectl describe pod nginx-deployment

# El servicio es una capa sobre el pod para hacerlo visible
service.yaml # https://hackernoon.com/kubernetes-101-2-3fdd96c7fe86

apiVersion: v1
kind: Service
metadata:
  labels:
    app: nginx
    role: web
    env: prod
  name: nginxservice
spec:
  ports:
    - port: 80
  selector:
    app: nginx
    role: web
    env: prod
  type: NodePort

kubectl create -f service.yaml
kubectl describe service nginxservice
minikube service nginxservice --url
http://192.168.99.100:30070/
http://192.168.99.100:30390/kk

# shell dentro de un contenedor de los del pod
kubectl get pods
kubectl exec nginx-deployment-2743933351-268sd -it sh

kubectl delete pod nginx-deployment-79d686f8f9-r2g77

kubectl get deploy

# actualización de la imagen de Docker y rollout undo
kubectl set image deployment/nginx-deployment nginx=nginx:1.9.1 ### Actualiza vers
kubectl describe deployments
kubectl get rs
http://192.168.99.100:30390/kk ###  vers 191
kubectl rollout undo deployment.v1.apps/nginx-deployment --to-revision=1
http://192.168.99.100:30390/kk ###  vers 179



# controlamos qué pods hay, escalado y deploy de nueva vers problematica
kubectl get pods
kubectl scale deployment.v1.apps/nginx-deployment --replicas=10
kubectl get pods
kubectl scale deployment.v1.apps/nginx-deployment --replicas=2
kubectl get pods

kubectl set image deployment.v1.apps/nginx-deployment nginx=nginx:verskk
kubectl describe deployments
kubectl get rs
kubectl rollout undo deployment.v1.apps/nginx-deployment --to-revision=1
kubectl get pod
kubectl get deploy


# Stateful Sets https://kubernetes.io/docs/concepts/workloads/controllers/statefulset/

kubectl get pods -w -l app=nginx # actualiza en directo
kubectl create -f statefulnginx.yml
kubectl get svc

for i in 0 1; do kubectl exec web-$i -- sh -c 'echo $(hostname) > /usr/share/nginx/html/index.html'; done
for i in 0 1; do kubectl exec -it web-$i -- curl localhost; done
kubectl delete pod -l app=nginx
for i in 0 1; do kubectl exec -it web-$i -- curl localhost; done
kubectl scale sts web --replicas=5
for i in 0 1 2 3 4; do kubectl exec -it web-$i -- curl localhost; done
for i in 0 1 2 3 4; do kubectl exec web-$i -- sh -c 'echo $(hostname) > /usr/share/nginx/html/index.html'; done
for i in 0 1 2 3 4; do kubectl exec -it web-$i -- curl localhost; done
minikube service nginx --url


#Ejemplo couchbase http://blog.kubernetes.io/2016/08/create-couchbase-cluster-using-kubernetes.html

kubectl run couchbase --image=arungupta/couchbase
    # Maestro
kubectl create -f https://github.com/arun-gupta/couchbase-kubernetes/blob/master/cluster/cluster-master.yml
kubectl get svc
minikube service couchbase-master-service #  usuario “Administrator” y la password “password”
    # Esclavo / worker
kubectl create -f https://github.com/arun-gupta/couchbase-kubernetes/blob/master/cluster/cluster-worker.yml
kubectl get rc
kubectl scale rc couchbase-worker-rc --replicas=3


# https://forums.couchbase.com/t/how-to-cbbackup-on-kubernetes-docker-issues/10544/2

# cbbackup
# https://github.com/couchbase/couchbase-cli


minikube ssh
minikube stop
minikube delete






# Traefik
# https://docs.traefik.io/user-guide/kubernetes/


export PATH=$HOME:$PATH
source <(kubectl completion bash)
minikube start
minikube status
kubectl get nodes
kubectl get pods
kubectl get services
minikube dashboard

# definiciones hosts para usuario local https://unix.stackexchange.com/questions/226260/hostaliases-file-with-an-ip-address#226318
echo 'g www.google.com' >> ~/.hosts
export HOSTALIASES=~/.hosts
wget g -O /dev/null


kubectl apply -f https://raw.githubusercontent.com/containous/traefik/master/examples/k8s/traefik-rbac.yaml
kubectl apply -f https://raw.githubusercontent.com/containous/traefik/master/examples/k8s/traefik-deployment.yaml
kubectl apply -f https://raw.githubusercontent.com/containous/traefik/master/examples/k8s/traefik-ds.yaml
# http://192.168.99.100:8080/dashboard/#/
# https://github.com/containous/traefik/tree/master/examples/k8s/traefik-deployment.yaml

kubectl --namespace=kube-system get pods
kubectl get services --namespace=kube-system
minikube ip
curl $(minikube ip)
# We expect to see a 404 response here as we haven't yet given Træfik any configuration.
# http://192.168.99.100:8080


# kubectl apply -f ejemplotraefik-quesos.yaml
# kubectl apply -f ejemplotraefik-quesos-ingress.yaml
# kubectl apply -f ejemplotraefik-path.yaml
# https://docs.traefik.io/user-guide/kubernetes/


# https://kubedex.com/nginx-ingress-vs-kong-vs-traefik-vs-haproxy-vs-voyager-vs-contour-vs-ambassador/
# http://traefik-ui.local:8081
kubectl apply -f https://raw.githubusercontent.com/containous/traefik/master/examples/k8s/cheese-deployments.yaml
kubectl apply -f https://raw.githubusercontent.com/containous/traefik/master/examples/k8s/cheese-services.yaml
kubectl apply -f https://raw.githubusercontent.com/containous/traefik/master/examples/k8s/cheese-ingress.yaml
echo 'traefik-ui.minikube' $(minikube ip) >> .hosts
echo 'cheddar.minikube' $(minikube ip) >> .hosts
export HOSTALIASES=~/.hosts
#https://addons.mozilla.org/en-US/firefox/addon/foxyproxy-standard/

# Docker + GlusterFS
# https://github.com/gluster/gluster-containers
# https://github.com/carmstrong/multinode-glusterfs-vagrant
# https://wiki.centos.org/SpecialInterestGroup/Storage/gluster-Quickstart

docker run -v /srv/gluster/etc/glusterfs:/etc/glusterfs:z -v /srv/gluster/var/lib/glusterd:/var/lib/glusterd:z -v /srv/gluster/var/log/glusterfs:/var/log/glusterfs:z -v /srv/gluster/sys/fs/cgroup:/sys/fs/cgroup:ro -d --privileged=true --net=host -v /dev/:/dev --name gluster gluster/gluster-centos


## Kubernetes Volumes
# https://kubernetes.io/docs/concepts/storage/volumes/
# https://kubernetes.io/docs/concepts/storage/persistent-volumes/


# Multi nodo

# https://coreos.com/kubernetes/docs/latest/kubernetes-on-vagrant.html
git clone https://github.com/coreos/coreos-kubernetes.git
cd coreos-kubernetes/multi-node/vagrant
vagrant box update
vagrant up
export KUBECONFIG="${KUBECONFIG}:$(pwd)/kubeconfig"
kubectl config set-cluster vagrant-multi-cluster --server=https://172.17.4.101:443 --certificate-authority=${PWD}/ssl/ca.pem
kubectl config set-credentials vagrant-multi-admin --certificate-authority=${PWD}/ssl/ca.pem --client-key=${PWD}/ssl/admin-key.pem --client-certificate=${PWD}/ssl/admin.pem
kubectl config set-context vagrant-multi --cluster=vagrant-multi-cluster --user=vagrant-multi-admin
kubectl config use-context vagrant-multi
kubectl cluster-info
kubectl get nodes
kubectl proxy
----> http://localhost:8001/ui





############################################################################################################

############ Vagrantfile de https://medium.com/@lizrice/kubernetes-in-vagrant-with-kubeadm-21979ded6c63

############################################################################################################

